import unittest
from unittest.mock import patch, Mock

from cast_ai.se.services.request_handle_svc import CustomHTTPError
from cast_ai.se.services.api_requests_svc import cast_api_get


class TestApiRequestsService(unittest.TestCase):
    # TODO: implement successful call
    # @patch('services.api_requests_svc.setup_logging')
    # @patch('services.api_requests_svc.get_api_key_headers')
    # @patch('services.request_handle_svc.get_api_key_headers')
    # @patch('services.request_handle_svc.handle_request')
    # def test_cast_api_get_success(self, mock_setup_logging, mock_get_api_key_headers1,
    #                               mock_get_api_key_headers2, mock_handle_request):
    #     # Arrange
    #     mock_setup_logging.return_value = None
    #     mock_get_api_key_headers1.return_value = {'Authorization': 'Bearer YOUR_API_KEY'}
    #     mock_get_api_key_headers2.return_value = {'Authorization': 'Bearer YOUR_API_KEY'}
    #     mock_response = Mock()
    #     mock_response.content.decode.return_value = '{"key": "value"}'
    #     mock_handle_request.return_value = mock_response
    #
    #     # Act
    #     result = cast_api_get('https://example.com/api')
    #
    #     # Assert
    #     self.assertEqual(result, {'key': 'value'})
    #     mock_setup_logging.assert_called_once()
    #     mock_get_api_key_headers1.assert_called_once()
    #     mock_get_api_key_headers2.assert_called_once()
    #     mock_handle_request.assert_called_once_with('https://example.com/api',
    #                                                 {'Authorization': 'Bearer YOUR_API_KEY'}, method='GET')

    @patch('misc_utils.setup_logging')
    @patch('services.api_requests_svc.get_api_key_headers')
    @patch('services.request_handle_svc.handle_request')
    def test_cast_api_get_failure(self, mock_setup_logging, mock_get_api_key_headers, mock_handle_request):
        # Arrange
        mock_setup_logging.return_value = None
        mock_get_api_key_headers.return_value = {'Authorization': 'Bearer YOUR_API_KEY'}
        mock_response = Mock()
        mock_response.content.decode.return_value = '{"error": "Something went wrong"}'
        mock_handle_request.side_effect = CustomHTTPError(response=mock_response)

        # Act & Assert
        with self.assertRaises(CustomHTTPError) as context:
            cast_api_get('https://example.com/api')

        self.assertEqual(str(context.exception),
                         'HTTP Error: 404 Client Error: Not Found for url: https://example.com/api')
        # mock_setup_logging.assert_called_once()
        # mock_get_api_key_headers.assert_called_once()
        # mock_handle_request.assert_called_once_with('https://example.com/api',
        # {'Authorization': 'Bearer YOUR_API_KEY'}, method='GET')


if __name__ == '__main__':
    unittest.main()



from abc import ABC, abstractmethod

from cast_ai.se.models.execution_status import ExecutionStatus


class CloudController(ABC):
    @abstractmethod
    def scale(self, node_count: int) -> ExecutionStatus:
        pass

    @abstractmethod
    def disable_autoscaler(self) -> ExecutionStatus:
        pass

    @abstractmethod
    def get_node_count(self) -> int:
        pass



import json
import logging
import subprocess

from azure.mgmt.containerservice import ContainerServiceClient

from cast_ai.se.models.cloud_confs import AksConfig
from cast_ai.se.models.execution_status import ExecutionStatus
from cast_ai.se.contollers.cloud_controller_svc import CloudController


class AKSController(CloudController):
    def __init__(self, azure_conf: AksConfig):
        try:
            self._conf = azure_conf
            self._logger = logging.getLogger(__name__)
            self._client = ContainerServiceClient(self._conf.credential, self._conf.subscription_id)
            self._cluster = self._client.managed_clusters.get(self._conf.resource_group, self._conf.cluster_name)
        except Exception as e:
            self._logger.critical(f"An error occurred during AKSController initialization: {str(e)}")
            raise RuntimeError(f"An error occurred during AKSController initialization: {str(e)}")

    def disable_autoscaler(self) -> ExecutionStatus():
        try:
            node_pool = next(pool for pool in self._cluster.agent_pool_profiles if pool.name == self._conf.node_pool)
            node_pool.enable_auto_scaling = False
            node_pool.count = 1
            update_operation = self._client.managed_clusters.begin_create_or_update(self._conf.resource_group,
                                                                                    self._conf.cluster_name,
                                                                                    self._cluster)
            return ExecutionStatus()
        except Exception as e:
            self._logger.exception(f"Error trying to disable autoscaling on nodepool {self._conf.node_pool}: {str(e)}")
            raise RuntimeError(f"Error trying to disable autoscaling on nodepool {self._conf.node_pool}: {str(e)}")

    # def get_node_count(self) -> int:
    #     try:
    #         node_pool = next(pool for pool in self._cluster.agent_pool_profiles if pool.name == self._conf.node_pool)
    #         return node_pool.count
    #     except Exception as e:
    #         self._logger.exception(f"Error trying to get node count on nodepool {self._conf.node_pool}: {str(e)}")
    #         raise RuntimeError(f"Error trying to get node count on nodepool {self._conf.node_pool}: {str(e)}")

    def get_node_count(self) -> int:
        try:
            command = f"kubectl --context {self._conf.k8s_context} get nodes -o=json"
            result = subprocess.run(command, shell=True, capture_output=True, text=True, check=True)
            nodes = json.loads(result.stdout)
            node_count = sum(1 for node in nodes.get('items', []) if node.get('metadata', {}).get('labels', {}).get(
                'kubernetes.azure.com/agentpool') == self._conf.node_pool)

            return node_count
        except subprocess.CalledProcessError as e:
            self._logger.exception(f"Error trying to get node count on nodepool {self._conf.node_pool}: {str(e)}")
            raise RuntimeError(f"Error trying to get node count on nodepool {self._conf.node_pool}: {str(e)}")

    def scale(self, node_count: int) -> ExecutionStatus:

        self._logger.info(f"{'-' * 70}[ Scaling AKS Node Pool to {node_count} nodes ]")
        try:
            node_pool = next(pool for pool in self._cluster.agent_pool_profiles if pool.name == self._conf.node_pool)
            node_pool.count = node_count
            update_operation = self._client.managed_clusters.begin_create_or_update(self._conf.resource_group,
                                                                                    self._conf.cluster_name,
                                                                                    self._cluster)
            return ExecutionStatus()
        except Exception as e:
            self._logger.exception(f"Error trying to set Demo Nodes capacity: {str(e)}")
            raise RuntimeError(f"Error trying to set Demo Nodes capacity: {str(e)}")



import json
import logging
import subprocess
import time
from datetime import datetime
from typing import Optional

from cast_ai.se.constants import CAST_NS
from cast_ai.se.misc_utils import get_get_deployments_command, validate_required_tools_exist
from cast_ai.se.models.execution_status import ExecutionStatus


class KubectlController:
    def __init__(self, context: str):
        self._logger = logging.getLogger(__name__)
        validate_required_tools_exist()
        self._cmd_prefix = f"kubectl --context {context}"
        self._context = context

    def trigger_cronjob(self, cronjob_name: str) -> ExecutionStatus:
        self._logger.info(f"{'-' * 70} Triggering {cronjob_name} cronjob")
        try:
            exec_timeout = 300
            exec_msg = "Cronjob still active..."
            job_name = self._create_cronjob_job(cronjob_name)
            for _ in range(exec_timeout):
                if self._has_job_completed(job_name):
                    exec_msg = ""
                    break
                else:
                    time.sleep(1)
            return ExecutionStatus(exec_msg)
        except Exception as e:
            self._logger.exception(f"An error occurred: {str(e)}")
            raise RuntimeError(f"An error occurred: {str(e)}")

    def _create_cronjob_job(self, cronjob_name) -> str:
        self._logger.info(f"Creating a job from {cronjob_name} in namespace {CAST_NS}")
        stamp = datetime.now().strftime("%d%m%H%M%S")
        job_name = f"{cronjob_name}-{stamp}"
        kubectl_cmd = ["kubectl", "--context", self._context, "create", "job", "-n", CAST_NS,
                       f"--from=cronjob/{cronjob_name}", job_name]
        result = subprocess.check_output(kubectl_cmd, text=True, shell=True)
        self._logger.debug(f"Command=[{' '.join(kubectl_cmd)} | Output=[{result.rstrip()}]")
        return job_name

    def scale_deployments(self, replica_count: int) -> ExecutionStatus:
        self._logger.info(f"{'-' * 70}[ Scaling Deployments to {replica_count} ]")
        try:
            # Get the list of deployments across all namespaces with namespace and name
            get_deployments_command = f"{self._cmd_prefix} {get_get_deployments_command(bool(not replica_count))}"
            result = subprocess.check_output(get_deployments_command, text=True, shell=True)
            deployments = result.split()
            self._logger.debug(f'Command=[{str(get_deployments_command)}] | Output=[{" ".join(deployments)}]')

            # Iterate over pairs of namespace and deployment name
            for i in range(0, len(deployments), 1):
                deployment_name = deployments[i]

                kubectl_cmd = (f"{self._cmd_prefix} scale --replicas={replica_count} deployment/{deployment_name} "
                               f"--namespace=default")
                result = subprocess.check_output(kubectl_cmd, text=True, shell=True)
                self._logger.debug(f'Command=[{" ".join(kubectl_cmd)}] | Output=[{result.rstrip()}]')
                self._logger.info(f"Deployment {deployment_name}[default] scaled to {replica_count} replicas.")
            return ExecutionStatus()
        except subprocess.CalledProcessError as e:
            self._logger.exception(f"Error executing kubectl command related to deployments scaling: {str(e)}")
            raise RuntimeError(f"Error executing kubectl command related to deployments scaling: {str(e)}")

    def _has_job_completed(self, job_name) -> Optional[bool]:
        try:
            command = f'{self._cmd_prefix} get job -n {CAST_NS} {job_name} -o json'
            result = subprocess.check_output(command, shell=True, text=True)
            job_info = json.loads(result)
            total_completions = job_info.get("spec", {}).get("completions", 1)
            succeeded = job_info["status"].get("succeeded", 0)
            failed = job_info["status"].get("failed", 0)
            if "succeeded" in job_info["status"] and succeeded + failed == total_completions:
                if job_info["status"].get("failed", 0) > 0:
                    self._logger.debug(f"The job '{job_name}' has completed with errors.")
                else:
                    self._logger.debug(f"The job '{job_name}' has completed successfully.")
                return True
            else:
                self._logger.debug(f"The job '{job_name}' is still running.")
                return False
        except subprocess.CalledProcessError as e:
            print(f"Error checking job status: {e}")
            return False

    def get_context_cloud_type(self) -> str:
        command = f"{self._cmd_prefix} cluster-info"
        try:
            result = subprocess.check_output(command, shell=True, text=True)
            if "eks.amazonaws.com" in result:
                self._logger.info("k8s cluster reflects it`s an EKS cluster")
                return "eks"
            elif "azmk8s.io" in result:
                self._logger.info("k8s cluster reflects it`s an AKS cluster")
                return "aks"
            self._logger.debug(f"k8s cluster reflects an unknown cluster type {result}")
            return ""
        except subprocess.CalledProcessError as e:
            self._logger.exception(f"Error executing/parsing kubectl command:{command} -=> {str(e)}")
            return ""



import json
import logging
import subprocess

from cast_ai.se.contollers.cloud_controller_svc import CloudController
from cast_ai.se.models.cloud_confs import GkeConfig
from cast_ai.se.models.execution_status import ExecutionStatus

from google.cloud import container_v1
from google.oauth2 import service_account


class GKEController(CloudController):
    def __init__(self, gke_conf: GkeConfig):
        try:
            self._conf = gke_conf
            self._logger = logging.getLogger(__name__)
            credentials = service_account.Credentials.from_service_account_file(
                self._conf.path,
                scopes=["https://www.googleapis.com/auth/cloud-platform"],
            )
            self._client = container_v1.ClusterManagerClient(credentials=credentials)
        except Exception as e:
            self._logger.critical(f"An error occurred during GKEController initialization: {str(e)}")
            raise RuntimeError(f"An error occurred during GKEController initialization: {str(e)}")

    def disable_autoscaler(self) -> ExecutionStatus:
        # TODO: ...implement this method
        return ExecutionStatus()

    def get_node_count(self) -> int:
        try:
            command = f"kubectl --context {self._conf.k8s_context} get nodes -o=json"
            result = subprocess.run(command, shell=True, capture_output=True, text=True, check=True)
            nodes = json.loads(result.stdout)
            node_count = sum(1 for node in nodes.get('items', []) if node.get('metadata', {}).get('labels', {}).get(
                'cloud.google.com/gke-nodepool') == self._conf.node_pool)
            return node_count
        except subprocess.CalledProcessError as e:
            self._logger.exception(f"Error trying to get node count on nodepool {self._conf.node_pool}: {str(e)}")
            raise RuntimeError(f"Error trying to get node count on nodepool {self._conf.node_pool}: {str(e)}")

    def _get_node_pool_path(self):
        cluster_path = f"projects/{self._conf.project_id}/locations/{self._conf.zone}/clusters/{self._conf.cluster_name}"
        node_pool_path = f"{cluster_path}/nodePools/{self._conf.node_pool}"
        return node_pool_path

    def scale(self, node_count: int) -> ExecutionStatus:
        try:
            node_pool_path = self._get_node_pool_path()
            update_request = container_v1.SetNodePoolSizeRequest(
                name=node_pool_path, node_count=node_count
            )
            response = self._client.set_node_pool_size(request=update_request)
            return ExecutionStatus()
        except Exception as e:
            self._logger.critical(f"An error occurred during GKEController initialization: {str(e)}")
            raise RuntimeError(f"An error occurred during GKEController initialization: {str(e)}")
